/*
 *  SingIt Lyrics Displayer
 *  Copyright (C) 2000 - 2003 Jan-Marek Glogowski <glogow@stud.fbi.fh-darmstadt.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "macros.h"

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <X11/Xlib.h>
#include <gtk/gtk.h>

#include <xmms/plugin.h>
#include <xmms/xmmsctrl.h>

#include "singit_main.h"
#include "singit_config.h"
#include "singit_config_dlg.h"
#include "singit_creator.h"
#include "singit_about.h"
#include "timeout.h"

#include <stdlib.h>
#include <gdk/gdkkeysyms.h>
#include <stdio.h>

#ifdef HAVE_ID3
#include "singit_id3dlg.h"
#endif

static void main_init(void);
static void main_cleanup(void);
static void main_about(void);
void main_config(void);
static void main_playback_start(void);
static void main_playback_stop(void);

SingitConfig *singit_config = NULL;
GuardedTimeout *timeout_tag = NULL;

VisPlugin singit_vp =
{
	NULL,
	NULL,
	0,
	SINGIT_VERSION_STRING,
	0,
	1,
	main_init, /* init */
	main_cleanup, /* cleanup */
	main_about, /* about */
	main_config, /* configure */
	NULL, /* disable_plugin */
	main_playback_start, /* playback_start */
	main_playback_stop, /* playback_stop */
	NULL, /* render_pcm */
	NULL  /* render_freq */
};

VisPlugin *get_vplugin_info(void)
{
	return &singit_vp;
}

static void main_init(void)
{
#ifdef DEBUG
	puts("main.c [main_init]");
#endif

#ifdef ENABLE_NLS
	setlocale (LC_ALL, "");
	bindtextdomain (PACKAGE, LOCALE_DIR);
#endif
	singit_main_show();
}

static void main_cleanup(void)
{
#ifdef DEBUG
	puts("main.c [main_cleanup]");
#endif

#ifdef HAVE_ID3
	singit_id3dlg_hide();
#endif
	singit_about_hide();
	singit_config_hide();
	singit_creator_hide();
	singit_main_hide();
	guarded_timeout_free(timeout_tag);
	singit_config_save(singit_config);
	singit_config_free(singit_config);
	singit_vp.disable_plugin(&singit_vp);
}

static void main_about(void)
{
#ifdef DEBUG
	puts("main.c [main_about]");
#endif

	singit_about_show();
}

void main_config(void)
{
#ifdef DEBUG
	puts("main.c [main_config]");
#endif

	singit_config_show();
}

static void main_playback_start(void)
{
#ifdef DEBUG
	puts("main.c [main_playback_start]");
#endif

	if (!singit_config->lyricPolling) {
		if (!timeout_tag) { timeout_tag = guarded_timeout_new(101, timeout_func, NULL); }
		else { guarded_timeout_attach(timeout_tag); }
	}
	else {
		singit_main_refresh();
	}
}

static void main_playback_stop(void)
{
#ifdef DEBUG
	puts("main.c [main_playback_stop]");
#endif

	if (singit_config) {
		if ((!singit_config->lyricPolling) && timeout_tag) {
			if (guarded_timeout_detach(timeout_tag, TRUE)) { timeout_tag = NULL; }
		}
	}
	else {
		if (guarded_timeout_detach(timeout_tag, TRUE)) { timeout_tag = NULL; }
	}
}
