/*
 *  SingIt Lyrics Displayer
 *  Copyright (C) 2000 - 2003 Jan-Marek Glogowski <glogow@stud.fbi.fh-darmstadt.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef __SINGIT_CONFIG_H__
#define __SINGIT_CONFIG_H__

typedef struct {

	// Changed by user - restored
	gboolean lyricPolling;      // Should user see new lyrics, even if no song is played ?
	gboolean toggleYZ;          // Instead of Z use Y (german keyboard "fix")
	gboolean showEmpty;         // Show the selection on empty lines
	gboolean rolling;           // Rolling is the old selection code, the selection "rolls" to the line
	gboolean autoResize;        // Resize the window automagically
	gboolean hideIfNotFound;    // Hide the lyric window, if no lyrics were found
        gchar* basePath;
	gchar* lyricExtension;
	gchar* lyricFontName;

	gchar       *backgroundColor;
	gchar       *normalTextColor;
	gchar       *normalBarColor;
	gchar       *progressTextColor;
	gchar       *progressBarColor;

	// From lyrics editor
	gboolean multiSelect;
	gboolean showHints;
	gboolean addSameTime;

	// Changed by program - restored
	gint singitWinPosX;
	gint singitWinPosY;
	gint creatorWinPosX;
	gint creatorWinPosY;
	gint configWinPosX;
	gint configWinPosY;
	gint singitWinSizeY;

	// config status
	gboolean isLoaded;
	guint    attachments;

} SingitConfig;

/* creates new config data includes attachment */
SingitConfig* singit_config_new(void);
/* frees config data even if there are attachments */
void singit_config_free(SingitConfig *config);

/* save config data */
void singit_config_save(SingitConfig *config);
/* load config data */
void singit_config_load(SingitConfig *config);

/* special save config data - just the editors options */
void singit_config_editor_save(SingitConfig *config);

/* in use - control mechanism */
/* if you use a config attach it, if none is attached config will be freed */
/* this will prevent trouble with freeing the config */

/* attach to a config file */
/* result == attach succeed ? TRUE : FALSE */
inline gboolean singit_config_attach(SingitConfig *config);
/* detach your config file - free it if there are no more attachments */
/* result == freed ? TRUE : FALSE */
inline gboolean singit_config_detach(SingitConfig *config, gboolean free);

#endif  // __SINGIT_CONFIG_H__
