/*
 *  SingIt Lyrics Displayer
 *  Copyright (C) 2000 - 2003 Jan-Marek Glogowski <glogow@stud.fbi.fh-darmstadt.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "macros.h"

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <X11/Xlib.h>
#include <gtk/gtk.h>
#include <pthread.h>

#ifdef HAVE_SCHED_SETSCHEDULER
#include <sched.h>
#endif

#include <xmms/plugin.h>
#include <xmms/xmmsctrl.h>

#include "singit_main.h"
#include "singit_config.h"
#include "singit_config_dlg.h"
#include "singit_creator.h"
#include "singit_about.h"
#ifdef HAVE_ID3
#include "singit_id3dlg.h"
#endif
#include "timeout.h"

#include <stdlib.h>
#include <gdk/gdkkeysyms.h>
#include <stdio.h>

static GtkWidget *singit_win = NULL;
static GtkWidget
		*singit_vbox,
			*scrlwnd, *clst,
			*singit_hbox,
			*singit_timestamp_status_bar,
			*singit_song_status_bar;
static GtkTooltips
		*singit_tooltips;

extern SingitConfig *singit_config;
extern GuardedTimeout *timeout_tag;
extern VisPlugin singit_vp;
extern void main_config(void);

// Changed by program - not restored
typedef struct {

        gboolean isPlaying;
	gboolean cleanedUp;
	gint maxLineWidth;
	gint lastRow;

	gint previous_song;
	gint previous_time;

	LSong *song;

	guint timestamp_context_id;
	guint song_context_id;

	gboolean lastFound;
	gboolean canChangeVisibility;
	gboolean isVisible;

} SingitStatus;

static SingitStatus singit_status;

inline LSong* singit_main_get_song(void)
{
	return singit_status.song;
}

static gboolean singit_main_change_visibility(gboolean visible, gboolean check)
{
	static gboolean shouldBeVisible = TRUE;
	if (!check) { shouldBeVisible = (!singit_config->hideIfNotFound || visible); }
	if (!singit_status.canChangeVisibility && !shouldBeVisible) { return FALSE; }
	if (shouldBeVisible == singit_status.isVisible) { return FALSE; }

#ifdef DEBUG
	puts("singit_main.c [singit_main_change_visibility]");
#endif

	if (shouldBeVisible) { gtk_widget_show(singit_win); }
	else { gtk_widget_hide(singit_win); }
	singit_status.isVisible = shouldBeVisible;
	return TRUE;
}

static void clst_jumpToTime(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	GList *item = NULL;
	gboolean isPlaying;
	if (!event) { return; }
	isPlaying = xmms_remote_is_playing(singit_vp.xmms_session);
	if (!isPlaying) {
		if (singit_status.song->prev_token == NULL) { return; }
		if (tLine(singit_status.song->prev_token) == row) { return; }
	}

#ifdef DEBUG
	printf("singit.c [clst_jumpToTime] :");
#endif

	if (singit_status.lastRow == row) {
		item = singit_status.song->first_token;
		while (item != singit_status.song->last_token) {
			if (tLine(item) == row) {
				if (!isPlaying) { xmms_remote_play(singit_vp.xmms_session); }
				xmms_remote_jump_to_time(singit_vp.xmms_session, tTime(item));
				return;
			}
			item = g_list_next(item);
		}
		if (tLine(item) == row) {
			if (!isPlaying) { xmms_remote_play(singit_vp.xmms_session); }
			xmms_remote_jump_to_time(singit_vp.xmms_session, tTime(item));
		}
	}
	else {
		singit_status.lastRow = row;
	}
#ifdef DEBUG
        puts(" Ok");
#endif
}

static void setStatusbarTime(gint time)
{
	static gint time_set = 0;
	gchar timeStamp[10];

	g_return_if_fail(GTK_WIDGET_REALIZED(singit_win));
	if ((time / 1000) == time_set) { return; }

#ifdef DEBUG
	printf("singit_main.c [createStatusbarMessages] : %i : ", time);
#endif

	timeStamp[9] = '\0';

	if (singit_status.timestamp_context_id != 0) {
		while (GTK_STATUSBAR(singit_timestamp_status_bar)->messages != NULL)
			gtk_statusbar_pop(GTK_STATUSBAR(singit_timestamp_status_bar),
				singit_status.timestamp_context_id);
	}
	else {
		singit_status.timestamp_context_id =
			gtk_statusbar_get_context_id(GTK_STATUSBAR(singit_timestamp_status_bar), "timestamps");
	}

	time_set = time / 1000;

	if (time_set > 0) {
		sprintf(timeStamp, " - %.2i:%.2i ", time_set / 60, time_set % 60);
		gtk_statusbar_push(GTK_STATUSBAR(singit_timestamp_status_bar),
			singit_status.timestamp_context_id, timeStamp);
	}
}

static void lyricsNotFound(void)
{
	gchar *fileNotFound, *nlbr[1][1];
	gint height;
	gint lbearing, rbearing, width, ascent, descent;

	singit_main_change_visibility(FALSE, FALSE);
	if (!singit_status.lastFound) { return; }
	else { singit_status.lastFound = FALSE; }

	gtk_clist_freeze(GTK_CLIST(clst));
	gtk_clist_clear(GTK_CLIST(clst));
	fileNotFound = g_strconcat(_("Lyrics not found"), NULL);
	nlbr[0][0] = fileNotFound;
	gtk_clist_append(GTK_CLIST(clst), nlbr[0]);
	if (singit_config->autoResize) {
		gdk_window_get_size (singit_win->window, &width, &height);
		singit_config->singitWinSizeY = height;
		gdk_text_extents(gtk_widget_get_style(clst)->font,
			_("Lyrics not found"),
			strlen(_("Lyrics not found")), &lbearing, &rbearing,
			&width, &ascent, &descent);
		gtk_window_set_default_size(GTK_WINDOW(singit_win),
			width + 50, ascent + descent + 50);
	}
	g_free(fileNotFound);
	gtk_clist_thaw(GTK_CLIST(clst));

	setStatusbarTime(0);
}

static gboolean buildLyricList(GtkWidget *lst, gchar *filename)
{
        gchar *buffer, *tmp, *fileName[2], *fileDir, *fileBase, *lbr[1][1];
        gint i = 0, strlength = -1, width, height;
	gboolean result = FALSE;

#ifdef DEBUG
	puts("singit_main.c [buildLyricList]");
#endif

	singit_status.maxLineWidth = -1;

	if (singit_config->basePath[0] == '~') {
		buffer = &singit_config->basePath[1];
		fileDir = g_strconcat(g_get_home_dir(), buffer, NULL);
	}
	else { fileDir = g_strdup(singit_config->basePath); }

	fileBase = strdup(g_basename(filename));
	buffer = strrchr(fileBase, '.');
	if (buffer != NULL) { buffer[1] = '\0'; }

	tmp = g_dirname(filename);
	fileName[0] = g_strconcat(fileDir, fileBase, singit_config->lyricExtension, NULL);
	fileName[1] = g_strconcat(tmp,
		"/", fileBase, singit_config->lyricExtension, NULL);
	if (buffer != NULL) { buffer[1] = '.'; }
	g_free(tmp);
	g_free(fileBase);
	g_free(fileDir);

#ifdef DEBUG
	printf("1. Constructed Filenames: %s\n                          %s\n", fileName[0], fileName[1]);
#endif

	if (!singit_status.song) { l_song_free(singit_status.song); }
	singit_status.song = l_song_new(filename);

	result = l_song_load_lyrics(singit_status.song, fileName[0]);
	if (!result) { result = l_song_load_lyrics(singit_status.song, fileName[1]); }
	if (!result) { result = l_song_load_lyrics(singit_status.song, filename); }
	g_free(fileName[1]);
	g_free(fileName[0]);

#ifdef HAVE_ID3
	l_song_get_id3_tag(singit_status.song, filename);
#endif

	if (!result) {
		lyricsNotFound();
		return FALSE;
	}
	else {
		setStatusbarTime(0);

		gtk_clist_freeze(GTK_CLIST(clst));
		gtk_clist_clear(GTK_CLIST(clst));

		while (singit_status.song->lyrics[i]) {
			strlength = strlen(&singit_status.song->lyrics[i][0]);
			if (strlength > 0) {  tmp = g_strndup(&singit_status.song->lyrics[i][0], strlength); }
			else { tmp = strdup("\0"); }
			lbr[0][0] = tmp;
			gtk_clist_append(GTK_CLIST(clst), lbr[0]);
			if (strlength > 0) {
		  		strlength = gdk_string_width(gtk_widget_get_style(clst)->font, tmp);
		  		if (strlength > singit_status.maxLineWidth) { singit_status.maxLineWidth = strlength; }
			}
			g_free(tmp);
			i++;
		}

#ifdef DEBUG
		printf("2. Most length line: %i\n\n", singit_status.maxLineWidth);
#endif
		if (singit_status.maxLineWidth < 150) { singit_status.maxLineWidth = 150; }
		if (singit_config->autoResize) {
			if (!singit_status.lastFound) {
				gtk_window_set_default_size(GTK_WINDOW(singit_win),
					singit_status.maxLineWidth + 45,
					singit_config->singitWinSizeY);
			}
			else {
				gdk_window_get_size (singit_win->window, &width, &height);
				gtk_window_set_default_size(GTK_WINDOW(singit_win), singit_status.maxLineWidth + 45, height);
			}
		}
		gtk_clist_thaw(GTK_CLIST(clst));
		singit_main_change_visibility(TRUE, FALSE);
		singit_status.lastFound = TRUE;
	}
	return TRUE;
}

static void selectLine(void)
{
	gchar *line;
	GList *item;

	gtk_clist_freeze(GTK_CLIST(clst));
	if (singit_status.song->prev_token != NULL)
		{ gtk_clist_unselect_row(GTK_CLIST(clst), tLine(singit_status.song->prev_token), 0); }
	if (!singit_config->showEmpty) {
		gtk_clist_get_text(GTK_CLIST(clst), tLine(singit_status.song->next_token), 0, &line);
		if (strlen(line) > 0) {
			gtk_clist_select_row(GTK_CLIST(clst), tLine(singit_status.song->next_token), 0);
			gtk_clist_moveto(GTK_CLIST(clst), tLine(singit_status.song->next_token), -1, 0.5, 0);
		}
	}
	else {
		gtk_clist_select_row(GTK_CLIST(clst), tLine(singit_status.song->next_token), 0);
		gtk_clist_moveto(GTK_CLIST(clst), tLine(singit_status.song->next_token), -1, 0.5, 0);
	}
	gtk_clist_thaw(GTK_CLIST(clst));
	singit_status.song->prev_token = g_list_previous(singit_status.song->next_token);

// * create timestamp statusbar *
	if (singit_status.song->prev_token != NULL) {
		if (g_list_next(singit_status.song->next_token) != NULL) {
			setStatusbarTime(tTime(g_list_next(singit_status.song->next_token)) -
				singit_status.previous_time + 1000);
		}
		else { setStatusbarTime(0); }
	}
	else {
		if (tTime(singit_status.song->first_token) > 0) {
			setStatusbarTime(tTime(singit_status.song->first_token) - singit_status.previous_time + 1000);
		}
		else {
			item = g_list_next(singit_status.song->first_token);
			if ((item) && (tTime(item) > 0))
				setStatusbarTime(tTime(item) - singit_status.previous_time + 1000);
		}
	}
}

static void goNewLyricLine(gint time, gint state)
{
	if ((state == 1) && (singit_status.song->prev_token == singit_status.song->last_token)) { return; }
	if ((state == -1) && (singit_status.song->next_token == singit_status.song->first_token)) { return; }

#ifdef DEBUG
	printf("singit_main.c [goNewLyricLine] : ");
#endif

	if (!singit_config->rolling) {
		singit_status.song->next_token = l_song_find_current_token(singit_status.song, time, state);
		if (singit_status.song->next_token == singit_status.song->first_token) {
			gtk_clist_moveto (GTK_CLIST(clst), tLine(singit_status.song->next_token), -1, 0.5, 0);
			singit_status.song->prev_token = NULL;
			if (time < tTime(singit_status.song->first_token)) {
				selectLine();
				return;
			}
		}
		singit_status.song->prev_token = g_list_previous(singit_status.song->next_token);
	}
	else {
		if (state == -1) {
			if (singit_status.song->prev_token != NULL) {
				if (g_list_previous(singit_status.song->prev_token) != NULL) {
					singit_status.song->next_token = g_list_previous(singit_status.song->prev_token);
					singit_status.song->prev_token = g_list_previous(singit_status.song->next_token);
				}
				else {
					singit_status.song->next_token = singit_status.song->first_token;
					singit_status.song->prev_token = NULL;
					selectLine();
					return;
				}
			}
			else {
				selectLine();
				return;
			}
		}
	}

	selectLine();
	singit_status.song->prev_token = singit_status.song->next_token;
	singit_status.song->next_token = g_list_next(singit_status.song->next_token);
#ifdef DEBUG
	puts("Ok");
#endif
}

static void set_status_filename(gchar *fileName)
{
	gchar *tmp;
	if (singit_status.song_context_id) {
		gtk_statusbar_pop(GTK_STATUSBAR(singit_song_status_bar), singit_status.song_context_id);
	}
	else {
		singit_status.song_context_id =
			gtk_statusbar_get_context_id(GTK_STATUSBAR(singit_song_status_bar), "filename");
	}
	tmp = g_strconcat(" ", fileName, NULL);
	gtk_statusbar_push(GTK_STATUSBAR(singit_song_status_bar), singit_status.song_context_id, tmp);
	gtk_tooltips_set_tip(singit_tooltips, singit_song_status_bar, fileName, NULL);
	g_free(tmp);
}

gint timeout_func(gpointer data)
{
	gchar *fileName = NULL;
	gint pos, time, status;

	g_return_val_if_fail(GTK_WIDGET_REALIZED(singit_win), 1);

        pos = xmms_remote_get_playlist_pos(singit_vp.xmms_session);
	time = xmms_remote_get_output_time(singit_vp.xmms_session);

	if (time < 0)
		{ return TRUE; }

	fileName = xmms_remote_get_playlist_file(singit_vp.xmms_session, pos);

	GDK_THREADS_ENTER();

        if ((pos != singit_status.previous_song) ||
		(fileName && l_song_changed(singit_status.song, fileName)))
       	{
               	singit_status.previous_song = pos;
		singit_status.previous_time = -1;
		if (fileName) {
			buildLyricList(clst, fileName);
			set_status_filename(fileName);
               	}
       	}
	if (fileName)
		{ g_free(fileName); }

	singit_status.previous_time = time;
	singit_creator_set_time(time);

	if (l_song_lyrics_found(singit_status.song)) {
		if ((status = inl_l_song_is_time_ok(singit_status.song, singit_status.previous_time))) {
#ifdef DEBUG
			printf(" %.2i:%.2i - ", time / 60000, (time / 1000) % 60);
#endif
			goNewLyricLine(time, status);
		}
		if (singit_status.song->next_token != NULL) {
			setStatusbarTime(tTime(singit_status.song->next_token) - time + 1000);
		}
	}

	if (gdk_window_is_visible(singit_win->window)) {
		gdk_window_get_position(singit_win->window, &(singit_config->singitWinPosX),
			&(singit_config->singitWinPosY));
	}

        GDK_THREADS_LEAVE();

        return TRUE;
}

void singit_main_refresh(void)
{
	gchar *fileName = NULL;
	gint pos;

#ifdef DEBUG
	puts("singit_main.c [singit_main_refresh]");
#endif

	if (singit_win) {
		if (singit_status.song) {
			if (!l_song_lyrics_changed(singit_status.song)) { return; }
		}
		pos = xmms_remote_get_playlist_pos(singit_vp.xmms_session);
		fileName = xmms_remote_get_playlist_file(singit_vp.xmms_session, pos);
               	if (fileName)
       	        {
			buildLyricList(clst, fileName);
			set_status_filename(fileName);
                       	g_free(fileName);
			singit_status.previous_time = -1;
        	}
	}
}

static gboolean singit_win_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
{
	gint volume;
	static gint jumpToSong = 0;

        switch(event->keyval) {
                case GDK_Escape: // exit plugin
			singit_vp.disable_plugin(&singit_vp);
                        break;
		case GDK_y:
			if (singit_config->toggleYZ)
				{ xmms_remote_playlist_prev(singit_vp.xmms_session); }
			break;
                case GDK_z:
			if (!singit_config->toggleYZ)
				{ xmms_remote_playlist_prev(singit_vp.xmms_session); }
                        break;
                case GDK_x:
                        xmms_remote_play(singit_vp.xmms_session);
                        break;
                case GDK_c:
			if (event->state & GDK_CONTROL_MASK) { main_config(); }
			else { xmms_remote_pause(singit_vp.xmms_session); }
                        break;
                case GDK_v:
                        xmms_remote_stop(singit_vp.xmms_session);
                        break;
                case GDK_b:
                        xmms_remote_playlist_next(singit_vp.xmms_session);
                        break;
		case GDK_e:
			if (event->state & GDK_CONTROL_MASK) { singit_creator_show(); }
			break;
		case GDK_a:
			if (event->state & GDK_CONTROL_MASK) { singit_about_show(); }
			break;
		case GDK_KP_Add:
			volume = xmms_remote_get_main_volume(singit_vp.xmms_session);
			if (volume < 100) { volume++; }
			xmms_remote_set_main_volume(singit_vp.xmms_session, volume);
			break;
		case GDK_KP_Subtract:
			volume = xmms_remote_get_main_volume(singit_vp.xmms_session);
			if (volume > 0) { volume--; }
			xmms_remote_set_main_volume(singit_vp.xmms_session, volume);
			break;
		case GDK_KP_Enter:
			if (jumpToSong == 0) { break; }
			if (xmms_remote_get_playlist_pos(singit_vp.xmms_session) != (jumpToSong-1))
				{ xmms_remote_set_playlist_pos(singit_vp.xmms_session, (jumpToSong-1)); }
		case GDK_KP_Decimal:
			jumpToSong = 0;
			break;
		case GDK_KP_0:
		case GDK_KP_1:
		case GDK_KP_2:
		case GDK_KP_3:
		case GDK_KP_4:
		case GDK_KP_5:
		case GDK_KP_6:
		case GDK_KP_7:
		case GDK_KP_8:
		case GDK_KP_9:
			jumpToSong = (jumpToSong*10) + (event->keyval - GDK_KP_0);
			if (jumpToSong > xmms_remote_get_playlist_length(singit_vp.xmms_session))
				{ jumpToSong = (event->keyval - GDK_KP_0); }
			break;
#ifdef HAVE_ID3
		case GDK_3:
			if (event->state & GDK_CONTROL_MASK) {
				singit_id3dlg_show(singit_status.song->id3tag,
					singit_status.song->fi_song->name);
			}
			break;
#endif
                default:
			// printf("%x - %x\n", event->state, event->keyval);
                        break;
        }

        return TRUE;
}

void singit_main_change_style(void)
{
	GtkRcStyle *rc_style;
	GdkFont *newFont;
	gchar *font_name;

	if (singit_win == NULL)
		{ return; }
	g_return_if_fail(GTK_WIDGET_REALIZED(singit_win));

	rc_style = gtk_rc_style_new ();

	gtk_clist_freeze(GTK_CLIST(clst));

// * Check if we could load a font with this name
	font_name = singit_config->lyricFontName;
	if ((newFont = gdk_font_load(font_name)) != NULL) {
		gdk_font_unref(newFont);
		rc_style->font_name = g_strdup(font_name);
		rc_style->fontset_name = g_strdup(font_name);
	}

	// * Ok - is there any logic to use these colors ??? *

// color of a selected row
	gdk_color_parse(singit_config->progressBarColor, &rc_style->bg[GTK_STATE_SELECTED]);
	rc_style->color_flags[GTK_STATE_SELECTED] |= GTK_RC_BG;

// color of the normal text
	gdk_color_parse(singit_config->normalTextColor, &rc_style->fg[GTK_STATE_NORMAL]);
	rc_style->color_flags[GTK_STATE_NORMAL] |= GTK_RC_FG;

// color of the a selected row text
	gdk_color_parse(singit_config->progressTextColor, &rc_style->fg[GTK_STATE_SELECTED]);
	rc_style->color_flags[GTK_STATE_SELECTED] |= GTK_RC_FG;

// background color of the row
	gdk_color_parse(singit_config->normalBarColor, &rc_style->base[GTK_STATE_NORMAL]);
	rc_style->color_flags[GTK_STATE_NORMAL] |= GTK_RC_BASE;

// "background" color of the space between the rows
	gdk_color_parse(singit_config->backgroundColor, &rc_style->base[GTK_STATE_ACTIVE]);
	rc_style->color_flags[GTK_STATE_ACTIVE] |= GTK_RC_BASE;

	gtk_widget_modify_style(clst, rc_style);

	gtk_rc_style_unref(rc_style);

// * Reseize the window *
	if (singit_config->autoResize == TRUE) {
		gtk_window_set_default_size(GTK_WINDOW(singit_win),
			gtk_clist_columns_autosize(GTK_CLIST(clst)) + 45, -1);
	}

	gtk_clist_thaw(GTK_CLIST(clst));
}

void singit_main_map_event (GtkWidget *widget, gpointer user_data)
{
	singit_status.canChangeVisibility = TRUE;
	singit_main_change_visibility(FALSE, TRUE);
}

void singit_main_unmap_event (GtkWidget *widget, gpointer user_data)
{
	singit_status.canChangeVisibility = FALSE;
}

void singit_main_resize_event (GtkWidget *widget, gpointer user_data)
{
	gint width, height;
/*#ifdef DEBUG
	puts("singit_main.c [singit_main_resize_event]");
#endif*/
	if (!singit_status.song) { return; }
	if (l_song_lyrics_found(singit_status.song))
		if (singit_config->autoResize) {
			gdk_window_get_size (singit_win->window, &width, &height);
			singit_config->singitWinSizeY = height;
		}
}

static void singit_main_init(void)
{
#ifdef DEBUG
	puts("singit_main.c [singit_main_init]");
#endif

	singit_status.cleanedUp = FALSE;
	singit_status.lastRow = -1;
	singit_status.previous_song = -1;
	singit_status.timestamp_context_id = 0;
	singit_status.song_context_id = 0;
	singit_status.lastFound = TRUE;
	singit_status.canChangeVisibility = TRUE;

	if (!singit_config_attach(singit_config)) {
		singit_config = singit_config_new();
		singit_config_load(singit_config);
	}

	singit_status.song = NULL;

	if (singit_config->lyricPolling) {
		if (!guarded_timeout_attach(timeout_tag)) {
			timeout_tag = guarded_timeout_new(101, timeout_func, NULL);
		}
	}
}

static void singit_main_cleanup(void)
{
	g_return_if_fail(!singit_status.cleanedUp);

#ifdef DEBUG
	puts("singit_main.c [singit_main_cleanup]");
#endif

	singit_status.cleanedUp = TRUE;

	if (singit_status.song != NULL) {
		l_song_free(singit_status.song);
		singit_status.song = NULL;
	}

	if (singit_win) {
		gdk_window_get_position(singit_win->window,
			&(singit_config->singitWinPosX),
			&(singit_config->singitWinPosY));
		singit_status.timestamp_context_id = 0;
	}

	if ((timeout_tag) && (singit_config->lyricPolling)) {
                if (guarded_timeout_detach(timeout_tag, TRUE)) { timeout_tag = NULL; }
	}

	singit_config_save(singit_config);
	if (singit_config_detach(singit_config, TRUE)) { singit_config = NULL; }

	singit_status.lastRow = -1;
	singit_status.previous_song = -1;
	singit_status.previous_time = -1;
	singit_status.lastFound = FALSE;
}

void singit_main_hide(void)
{
	if (!singit_win) return;

#ifdef DEBUG
	puts("singit_main.c [singit_main_hide]");
#endif

	singit_main_cleanup();
	gtk_widget_destroy(singit_win);
}

static gint singit_main_delete_event( GtkWidget *widget, GdkEvent  *event, gpointer data )
{
	singit_main_cleanup();
	return (FALSE);
}

void singit_main_show(void)
{
	if (singit_win) return;

#ifdef DEBUG
	puts("singit_main.c [singit_main_show]");
#endif

	singit_main_init();
	singit_win = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_set_title(GTK_WINDOW(singit_win), SINGIT_VERSION_STRING);
	gtk_window_set_policy(GTK_WINDOW(singit_win), TRUE, TRUE, TRUE);
	gtk_window_set_default_size(GTK_WINDOW(singit_win), 230, 110);
	gtk_widget_set_events(singit_win, GDK_MAP | GDK_UNMAP);
	gtk_signal_connect(GTK_OBJECT(singit_win), "destroy",
		GTK_SIGNAL_FUNC(gtk_widget_destroyed), &singit_win);
	gtk_signal_connect(GTK_OBJECT(singit_win), "delete_event",
		GTK_SIGNAL_FUNC(singit_main_delete_event), NULL);
	gtk_signal_connect(GTK_OBJECT(singit_win), "key-press-event",
		GTK_SIGNAL_FUNC(singit_win_keypress), NULL);
	gtk_signal_connect(GTK_OBJECT(singit_win), "map-event",
		GTK_SIGNAL_FUNC(singit_main_map_event), NULL);
	gtk_signal_connect(GTK_OBJECT(singit_win), "unmap-event",
		GTK_SIGNAL_FUNC(singit_main_unmap_event), NULL);
	gtk_signal_connect(GTK_OBJECT(singit_win), "size-allocate",
		GTK_SIGNAL_FUNC(singit_main_resize_event), NULL);
	gtk_container_border_width(GTK_CONTAINER(singit_win), 5);

	singit_vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(singit_win), singit_vbox);
	gtk_widget_show(singit_vbox);

	scrlwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrlwnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_widget_set_usize(scrlwnd, 180, 80);
	gtk_box_pack_start(GTK_BOX(singit_vbox), scrlwnd, TRUE, TRUE, 0);
	gtk_widget_show (scrlwnd);

	clst = gtk_clist_new(1);
	gtk_signal_connect(GTK_OBJECT(clst), "select_row",
		GTK_SIGNAL_FUNC(clst_jumpToTime), NULL);
	gtk_clist_set_selection_mode(GTK_CLIST(clst), GTK_SELECTION_SINGLE);
	gtk_clist_set_reorderable (GTK_CLIST(clst), FALSE);
	gtk_container_add(GTK_CONTAINER(scrlwnd), clst);
	gtk_clist_set_column_width(GTK_CLIST(clst), 0, 1);
	gtk_widget_show(clst);

	singit_hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(singit_vbox), singit_hbox, FALSE, FALSE, 1);
	gtk_widget_show(singit_hbox);

	singit_timestamp_status_bar = gtk_statusbar_new();
	gtk_box_pack_start (GTK_BOX(singit_hbox), singit_timestamp_status_bar, FALSE, FALSE, 0);
	gtk_widget_show(singit_timestamp_status_bar);

	singit_song_status_bar = gtk_statusbar_new();
	gtk_box_pack_start (GTK_BOX (singit_hbox), singit_song_status_bar, TRUE, TRUE, 0);
	gtk_widget_show(singit_song_status_bar);

	singit_tooltips = gtk_tooltips_new ();
	gtk_tooltips_enable(singit_tooltips);

	// * Normally a window is created when it's shown first,
	// * so if not shown we must explicit realize it to get a GDK_WINDOW *
	if (!singit_config->hideIfNotFound) {
		gtk_widget_show(singit_win);
		singit_status.isVisible = TRUE;
		singit_main_change_visibility(TRUE, TRUE);
	}
	else {
		gtk_widget_realize(singit_win);
		singit_status.isVisible = FALSE;
	}
	singit_main_change_style();

	singit_status.timestamp_context_id =
		gtk_statusbar_get_context_id(GTK_STATUSBAR(singit_timestamp_status_bar), "timestamps");
	gtk_statusbar_push(GTK_STATUSBAR(singit_timestamp_status_bar),
		singit_status.timestamp_context_id, "         ");

	if ((singit_config->singitWinPosX < 0) || (singit_config->singitWinPosY < 0)) {
		gtk_window_set_position(GTK_WINDOW(singit_win), GTK_WIN_POS_MOUSE);
	}
	else {
		gdk_window_move(singit_win->window, singit_config->singitWinPosX, singit_config->singitWinPosY);
	}
}
