/*
 *  SingIt Lyrics Displayer
 *  Copyright (C) 2000 - 2002 Jan-Marek Glogowski <glogow@stud.fbi.fh-darmstadt.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>
#include "config.h"
#ifdef DEBUG
#include <stdio.h>
#endif
#include "timeout.h"

GuardedTimeout* guarded_timeout_new(guint interval, GSourceFunc function, gpointer data)
{
	GuardedTimeout* result;

#ifdef DEBUG
	puts("timeout.c [guarded_timeout_new]");
#endif

	result = g_malloc(sizeof(GuardedTimeout));

	result->timeoutId = g_timeout_add(interval, function, data);
	result->attachments = 1;

	return result;
}

void guarded_timeout_free(GuardedTimeout* timeout)
{
	if (!timeout) { return; }

#ifdef DEBUG
	puts("timeout.c [guarded_timeout_free]");
#endif

	if (timeout->timeoutId) { gtk_timeout_remove(timeout->timeoutId); }
	timeout->timeoutId = 0;

	g_free(timeout);
}

gboolean guarded_timeout_attach(GuardedTimeout *timeout)
{
#ifdef DEBUG
	printf("timeout.c [guarded_timeout_attach] : ");
#endif

	if (timeout != NULL) {
		timeout->attachments++;
#ifdef DEBUG
		puts("Attached");
#endif
		return TRUE;
	}
#ifdef DEBUG
	puts("Failed");
#endif
	return FALSE;
}

gboolean guarded_timeout_detach(GuardedTimeout *timeout, gboolean free)
{
#ifdef DEBUG
	printf("timeout.c [guarded_timeout_detach] : ");
#endif

	if (!timeout) { return TRUE; }
	timeout->attachments--;
	if ((timeout->attachments == 0) && (free)) {
#ifdef DEBUG
		puts("Freed");
#endif
		guarded_timeout_free(timeout);
		return TRUE;
	}
#ifdef DEBUG
	puts("Detached");
#endif
	return FALSE;
}
